;*********************************************************************
; Local params and values

write_buffer_size db ?
next_address      dw ?


ZFL_CTL_PORT  EQU 218H
ZFL_DATB_PORT EQU 219H
ZFL_DATW_PORT EQU 21AH
WINDOWBASE    EQU 0E000H
;*********************************************************************
; flash_cmd
;
; command the atmel flash to perform func in al
flash_cmd proc near
			push ds
			push si
			push ax
			push WINDOWBASE
			pop  ds

			; prep commands
			mov  ax, 0AAAAh
			mov  si, 05555h
			shl  si, 1
			mov  ds:[si], ax
			mov  si, 0AAAAh
			shl  si, 1
			mov  ax, 05555h
			mov  ds:[si], ax

			; now the real command
			pop  ax
			mov  si, 05555h
			shl  si, 1
			mov  ds:[si], ax
			pop si
			pop ds
			ret
flash_cmd endp

;*********************************************************************
; flash_banner
; Print an opening banner for this flash burner
;
Msg	db "Atmel BUR Programmer", C_CR, C_LF, 0

flash_banner proc near
			push es
			push cs
			pop es
			mov di, offset Msg
			xor cx,cx
			call SerSend
			pop es
			ret
flash_banner endp

;*********************************************************************
; flash_confirm
; Procedure to confirm the flash is consistent with the programmer
; code embedded herein. Return is zero if the burner cannot burn the
; flash it found. Return is zero if good to erase and burn.
;
; Proc should print a flash specific message if inconsistent
;
;	return: 	ax=0: flash good to go
;           ax=non-zero: flash not consistent with programmer
;

; Local messages
fc_query_error   db C_ESC, "Atmel signature not found, device returned: ", 0
fc_query_success db "Atmel signature found, device returned: ", 0

flash_confirm proc near
			push ds
			push es
			push cs
			pop  es
			push WINDOWBASE
			pop  ds

			; configure mem_cs0, E000, 64Kbyte Window, first page
			WRITE_ZFL32 26H, WINDOWBASE*16 ; map it to e000
			WRITE_ZFL32 2AH, 0FFFFH  	; 64K window
			WRITE_ZFL32 2EH, 0       	; first page

			; configure cs0 for read/write, 16 bit mode
			READ_ZFL08  5AH
			and al, 0EEH         ; CS0 now read/write access and 16 bit
			WRITE_ZFL08 5AH, al

			; Get chip id
			mov  ax, 90h         ; get id function
			call flash_cmd
			xor  si, si
			mov  eax, ds:[si]    ; get the id
			push eax
			mov  ax, 0F0h        ; end identification mode
			call flash_cmd
			pop eax
			mov ebx, eax

			; Now see if its the correct chip
			and ebx, 0fffcffffh
			cmp ebx, 00084001Fh
			jz  fc_gotit

			; Didn't match
			mov di, offset fc_query_error
			xor cx, cx
			call SerSend
			call SerOut32
			call CRLF
			mov ax, 1
			jmp short fc_quit

fc_gotit:
			; Got the correct ID
			mov di, offset fc_query_success
			xor cx, cx
			call SerSend
			call SerOut32
			call CRLF
			xor ax, ax
fc_quit:
			pop es
			pop ds
			ret
flash_confirm endp

;*********************************************************************
; flash_erase
; Procedure to erase the flash. Return in AL is zero if failed, non-zero
; for success.
;
;	return: 	ax=0: success
;           ax=non-zero: failed
;
fe_erase_msg   db "Erasing flash: may take as long as 10 seconds", C_CR, C_LF, 0
fe_erase_cmpt  db "Flash Erased", C_CR, C_LF, 0
flash_erase_segment proc near
			; save seg registeres
			push ds
			push es
			push fs
			push di
			push si

			; Send erase message
			push cs
			pop es
			mov di, offset fe_erase_msg
			xor cx, cx
			call SerSend

			; Chip erase commands
			mov ax, 080h
			call flash_cmd
			mov ax, 010h
			call flash_cmd

			; Erase underway. Timeout in 1 seconds
			mov ecx, 10000h
			push WINDOWBASE
			pop fs
			xor si, si
fe_loop:
			READ_ZFL08 5Ah
			cmp dword ptr fs:[si], 0FFFFFFFFh
			je fe_fini
			dec ecx
			jnz fe_loop

fe_fini:
			; Send erase message
			mov di, offset fe_erase_cmpt
			xor cx, cx
         mov cs:[next_address], cx
			call SerSend

			; all done
			xor ax, ax
			pop si
			pop di
			pop fs
			pop es
			pop ds
			ret

flash_erase_segment endp

;*********************************************************************
; flash_burn_pkt
;
;	return: 	ax=0: success
;           ax=non-zero: failed
;
fb_failed_readback_msg   db C_ESC, "Failed readback during byte write at address: " , 0

flash_burn_pkt proc near
			; save regs
			push bx
			push cx
			push ds
			push es
			push fs
			push si
			push di

			; setup regs for accessing the pkt buffer and region of flash
			push cs
			pop  ds
			push cs
			pop  es
			push WINDOWBASE
			pop  fs
			mov  cx, PKT_SIZE - 3
			mov  di, cs:[next_address]
			mov  si, offset dbuf + 2

fb_cx_lp:
			mov   ax, 0A0A0h                   ; code to program word
			call  flash_cmd                  ; tell the flash
			mov   ax, ds:[si]						; get data word
			mov   fs:[di], ax						; put it out there
			mov   bx, ax

			; little loop to confirm the write cycle took
			push  cx
			mov   cx, 500
fb_WaitWriting:
			in al, 80H		    					; perform an ISA cycle
			cmp	word ptr fs:[di], bx			; see if data is written
			je	   fb_ByteOk                  ; jump if done
			loopnz fb_WaitWriting

			; Fall through is an error
			xor cx, cx
			push di
			mov  di, offset fb_failed_readback_msg
			call SerSend
			pop  ax
			call SerOut16
			mov  ax, 1
			jmp  short fb_exit

fb_ByteOk:
			pop cx
			add si, 2
			add di, 2
			sub cx, 2
			jnz fb_cx_lp

			; Pkt is now completely written. Save next address for next time
			mov cs:[next_address], di
			; Indicate no error
			xor ax, ax

fb_exit:
			; We are outa here. Restore regs for caller
			pop di
			pop si
			pop fs
			pop es
			pop ds
			pop cx
			pop bx
			ret
flash_burn_pkt endp

