// machztrm.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include "crctab.c"

//////////////////////////////////////////////////////////////////////////////
#define SOH 0x01
#define EOT 0x04
#define ACK 0x06
#define NAK 0x15
#define CAN 0x18
#define CTRLZ 0x1A
#define ESC 0x1B

//////////////////////////////////////////////////////////////////////////////
// Structure required to support ymodem protocol
typedef struct 
{
   char hdr;
   char frame;
   char nframe;
   char data[128];
   char crc_hi;
   char crc_lo;
} ymodem_struct;

// My simple minded protocol
typedef struct
{
   char start;
   char serial;
   char data[64];
   char csum;
} sp_pkt;

//////////////////////////////////////////////////////////////////////////////
char frame_num = 0;
HANDLE hCom = NULL;

//////////////////////////////////////////////////////////////////////////////
int  echo_monitor_string(int count = 1500);
int  echo_chars(char* buf = NULL, int buf_size = 0);
int  send_ymodem_file ( char * filename, int dls_address );
int  setup_comms ( int );
int  get_download_address ( int * dls_address );
int  send_file_simple_protocol(char * file_name);
int  send_sp_packet ( sp_pkt& );

//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
   DWORD dnum;
   int  tmp, i, dls_address, done;
	char *pcCommPort = "COM1";
   char cmd[100];

   // Confirm arguments
   if ( argc != 3 ) 
   {
      printf("Proper command for invoking this application is:\n\n");
      printf("   machztrm bur_pgm_name bios_filename\n\n");
      printf("bur_pgm_name must identify the file (with extension) for\n");
      printf("   MachZ BUR application for programming your flash\n");
      printf("bios_filename must identify the file to be burnt into the flash\n");
      return 1;
   }

	hCom = CreateFile( pcCommPort,
                     GENERIC_READ | GENERIC_WRITE,
                     0,    // comm devices must be opened w/exclusive-access
                     NULL, // no security attributes
                     OPEN_EXISTING, // comm devices must use OPEN_EXISTING
                     0,    // not overlapped I/O
                     NULL  // hTemplate must be NULL for comm devices
                     );  

	if (hCom == INVALID_HANDLE_VALUE) 
	{
		// Handle the error.
		printf ("CreateFile failed with error %d.\n", GetLastError());
		return (1);    
	}
   printf ("Serial port %s successfully reconfigured.\nPower up the target with the A23 jumper installed now\n", pcCommPort);

   // Setup the communications
   if( ( tmp = setup_comms(CBR_9600) ) != 0 )
   {
      // Unable to setup communications. print error code and exit
      printf("Error configuring serial port: %d\n", tmp);
      return 1;
   }

   // Flush the receive queue and wait for the monitor to wake up
   echo_chars();
   if ( !echo_monitor_string(3000) )
   {
      printf("Timeout waiting for target reset string. Please try again\n");
      return 1;
   }

   // Pump up the baud rate for us both
   sprintf(cmd, "speed  56  0  \xA\xD\xA\xD");
   WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
   echo_chars();
   setup_comms(CBR_56000);
   echo_chars();
   Sleep (2000);

   // Send a cr/lf and see if monitor comes back
   strcpy(cmd, "\xA\xD");
   WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
   echo_monitor_string();

   // Get the download address from the MachZ
   if ( !get_download_address(&dls_address) )
      printf("Error getting download address\n" );

   else if (send_ymodem_file(argv[1], dls_address))
   {
      // Send it the loadbios.com file
      Sleep(1000);
      echo_monitor_string();

      // Tell it to execute the application loaded
      sprintf(cmd, "g %x:0\xA\xD", dls_address);
      WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
      for ( done=0, tmp=0, i=0; tmp<640 /*50*/ && !done; tmp++ ) 
      {
         Sleep (200);
         int num_chars = echo_chars(cmd, sizeof(cmd));
         for ( i=0; i<=num_chars; i++ ) 
         {
            if ( cmd[i] == ESC || cmd[i] == ACK || cmd[i] == NAK )
            {
               done++;
               break;
            }
         }
      }

      // Lets try to download a file
      if ( cmd[i] != ESC ) send_file_simple_protocol(argv[2]);
      else echo_monitor_string();

      // Dump bytes at the location
      sprintf(cmd, "db %x:0\xA\xD", dls_address);
      WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
      Sleep(1000);
      echo_chars();

      // Dump next page at the location
      sprintf(cmd, "d\xA\xD");
      WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
      Sleep(1000);
      echo_chars();
   }
   // Drop baud rate to 9600
   sprintf(cmd, "speed   96  0  \xA\xD\xA\xD");
   WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
   echo_chars();
   setup_comms(CBR_9600);
   Sleep (1000);
   echo_chars();
   return (0);
}

//////////////////////////////////////////////////////////////////////////////
int setup_comms ( int br )
{
	DCB dcb;  
   COMMTIMEOUTS timeouts;

	// We will build on the current configuration, and skip setting the size
	// of the input and output buffers with SetupComm.
	if (!GetCommState(hCom, &dcb)) 
	{
		// Handle the error.
		printf ("GetCommState failed with error %d.\n", GetLastError());
		return (2);    
   }
   // Fill in the DCB: baud rate from arg, 8 data bits, no parity, and 1 stop bit.
   dcb.BaudRate = br;            // set the baud rate
   dcb.ByteSize = 8;             // data size, xmit, and rcv
   dcb.Parity = NOPARITY;        // no parity bit
   dcb.StopBits = ONESTOPBIT;    // one stop bit
   dcb.fNull = 0;
   if (!SetCommState(hCom, &dcb)) 
   {
      // Handle the error.
      printf ("SetCommState failed with error %d.\n", GetLastError());
      return (3);    
   }

   if ( !GetCommTimeouts( hCom, &timeouts) )
   {
      printf("Error: GetCommTimeouts reported error number %d.\n", GetLastError());
      return 4;
   }
   timeouts.ReadIntervalTimeout = 10;
   timeouts.ReadTotalTimeoutMultiplier = 10;
   if ( !SetCommTimeouts(hCom, &timeouts) )
   {
      printf("Error: SetCommTimeouts reported error number %d.\n", GetLastError());
      return 5;
   }
   return 0;
}

//////////////////////////////////////////////////////////////////////////////
int get_download_address ( int* dls_address )
{
   char *dls = "dls\xA\xD";
   char buf[100];
   DWORD dnum;
   int retval = 0;

   // Flush whatever is in receive queue to display
   echo_chars();

   // Send the dls command
   WriteFile(hCom, dls, strlen(dls), &dnum, NULL);

   // Echo the monitor string preceding the value
   if ( echo_monitor_string() )
   {
      // Last character received was a colon, ready for the dls value
      Sleep (50);
      if ( echo_chars(buf, sizeof(buf) ) != 0 )
      {
         if ( sscanf(buf,"%x", dls_address) == 1 )
            retval++;
         else
            printf("Unable to convert MachZ dls address: %s\n", buf);
      }
   }
   return retval;
}

//////////////////////////////////////////////////////////////////////////////
// echo_monitor_string
// Retrieve characters until a colon or a timeout occurs. Return non-zero 
// if characters were received, zero if not and timeout expires
int echo_monitor_string(int count)
{
   DWORD dnum=1;
   char chr = 0;
   int retval = 1;


   // read characters from the port until we see a ":"
   while (chr != ':' && retval != 0 )
   {
      ReadFile(hCom, &chr, 1, &dnum, NULL);
      if ( dnum == 1 ) 
      {
         printf("%c", chr);
         count = -1;
      }
      else if ( ( count > 0 ) && (--count == 0 ) ) retval = 0;
   }
   return retval;
}


//////////////////////////////////////////////////////////////////////////////
// echo_chars will echo characters to the display and to the buffer 
// until the receiver times out or the buffer is saturated. Return value
// is the number of characters pumped.
int echo_chars(char* buf, int buf_size)
{
   DWORD dnum=1;
   char chr = 0;
   int idx = 0, count = 0;

   // read characters from the port until the queue is flushed
   while(dnum)
   {
      ReadFile(hCom, &chr, 1, &dnum, NULL);
      if ( dnum == 1 ) 
      {
         printf("%c", chr);
         count++;
         if ( buf_size != 0 && buf != NULL ) 
         {
            buf[idx++] = chr;
            if ( idx >= buf_size ) break;
         }
      }
   }
   return count;
}


//////////////////////////////////////////////////////////////////////////////
// Compute the CRC for the ymodem data structure
void compute_crc ( ymodem_struct * d )
{
   unsigned short c = 0;

   for ( int i=0; i<sizeof(d->data); i++ )
      c = (unsigned short)(updcrc((0xFF&d->data[i]),c));
   c = (unsigned short)updcrc(0,updcrc(0,c));
   d->crc_hi = (char)((c & 0xFF00)>>8);
   d->crc_lo = (char)(c & 0x00FF);
}


//////////////////////////////////////////////////////////////////////////////
int send_frame(ymodem_struct* sptr)
{
   DWORD dret;
   int done = 0, retval = 0, sendcnt=0, i;
   char chr;

   // First, compute the crc for the packet
   compute_crc(sptr);
   // Flush receive buffer to screen
   echo_chars();
   while ( sendcnt++ < 5 && !done )
   {
      // Send the packet
      if (!WriteFile(hCom, sptr, sizeof(*sptr), &dret, NULL))
         printf("send_frame: WriteFile Failed\n");
      else if ( dret != sizeof(*sptr) )
         {
            retval = 0;
            printf("Error writing ymodem frame\n");
         }
      else 
      {
         Sleep(150);
         dret = 0;
         // Try (up to 10 times) to get a char from receiver.
         for (i=0; i<10 && dret == 0; i++ )
         {
            if (!ReadFile(hCom, &chr, 1, &dret, NULL))
            {
               printf("send_frame:: ReadFile Failed error = %ld\n",
                        GetLastError());
               done++;
            }
               // If an ACK, we be done with this frame
            else if ( dret == 1 && chr == ACK ) 
            {
               printf("*");
               retval = ++done;
            }
            else if (dret == 1 && chr == NAK )
               printf("/");
            else if (dret == 1)
               printf("0x%2.2x ", chr);
            else
               printf("1");
         } // End for
      }
   }
   if ( sendcnt == 5 ) 
      printf("Ymodem send_frame retry count exceeded\n");

   return retval;
}


//////////////////////////////////////////////////////////////////////////////
void send_eot ( void )
{
   int sendcnt = 0, done = 0, i;
   DWORD dret;
   char chr=EOT, crx;

   // y-modem protocol requires we send an EOT until an ACK is received.
   while ( sendcnt++ < 10 && !done )
   {
      // Send the eot
      WriteFile(hCom, &chr, 1, &dret, NULL);
      // Try (up to 10 times) to get a char from receiver.
      for (i=0; i<10 && !done; i++ )
      {
         if (ReadFile(hCom, &crx, 1, &dret, NULL))
         {
            // If an ACK, we be done
            if ( dret == 1 && crx == ACK ) 
            {
               printf("E");
               done++;
            }
            else if ( dret == 1 )
               printf("F");
         }
      } // End for
   }
   echo_chars();
}


//////////////////////////////////////////////////////////////////////////////
int send_ymodem_file ( char* filename, int dls_address )
{
   ymodem_struct d;
   int i, retval = 0;
   FILE* file;
   fpos_t pos;
   DWORD dnum;
   char cmd[100];

   // Open the file to be sent
   file = fopen(filename, "rb");
   if ( !file )
   {
      printf("Error opening file: process aborted\n");
      return 0;
   }
   // Get file size
   fseek(file, 0, SEEK_END);
   fgetpos(file, &pos);
   fseek(file, 0, SEEK_SET);

   // Format the yload command
   sprintf(cmd, "yload %x:0\xA\xD", dls_address);
   WriteFile(hCom, cmd, strlen(cmd), &dnum, NULL);
   Sleep(1000);
   echo_chars();

   // Initialize the data structure to zero
   for ( i=0; i<sizeof(d.data); i++ ) d.data[i] = 0; 

   // Set all params for block 0
   d.hdr = SOH;
   d.frame = frame_num = 0;
   d.nframe = (char)~d.frame;
   strcpy(d.data, filename);
   sprintf(&d.data[strlen(filename)+1], "%ld", pos);
   if ( send_frame(&d) )
   {
      // So far, so good. Send the file until EOF encountered.
      int itmp = 10, done = 0;
      while ( itmp && !done)
      {
         // Initialize the data structure
         for ( i=0; i<sizeof(d.data); i++ ) 
            d.data[i] = CTRLZ; 
         d.hdr = SOH;
         d.frame = ++frame_num;
         d.nframe = (char) ~d.frame;
         // Get the next frame from the file
         itmp = fread(d.data, 1, sizeof(d.data), file);
         if ( itmp == 0 && !feof(file) )
         {
            printf("Error reading file %s: %d\n", filename, ferror(file));
            break;
         }
         else if ( itmp < sizeof(d.data) )
            retval = ++done;
         // Make it go
         if ( !send_frame(&d) ) 
            done++;
      }
   }
   else retval = 0;

   // Send EOT
   send_eot();
   echo_chars();
   fclose(file);

   // And send a null packet marking end of data
   for ( i=0; i<sizeof(d.data); i++ ) d.data[i] = 0; 
   // Set all params for block 0
   d.hdr = SOH;
   d.frame = 0;
   d.nframe = (char)~d.frame;
   if ( !send_frame(&d) ) printf("Error sending final null pkt 0\n");

   return retval;
}

//////////////////////////////////////////////////////////////////////////////
int  send_file_simple_protocol(char * file_name)
{
   sp_pkt s;
   FILE * file;
   int i,done, early, itmp;
   DWORD dret = 1;
   char chr;

   // Open the file to be sent
   file = fopen(file_name, "rb");
   if ( !file )
   {
      printf("Error opening file %s: process aborted\n", file_name);
      return 1;
   }

   // Zero out the serial number
   frame_num = 0;
   // Flush all in the receive queue
// while (dret) ReadFile(hCom, &chr, 1, &dret, NULL);
   // Now wait til we get a single ACK
   i = 0;
   do
   {
      ReadFile(hCom, &chr, 1, &dret, NULL);
   } while ( i++ < 100 && chr != ACK );

   // Loop to transfer the entire file in packets
   printf ("\n");
   done = early = 0;
   do {
      // Read data from the file
      itmp = fread(s.data, 1, sizeof(s.data), file);
      if ( itmp == 0 )
      {
         if ( !feof(file) ) printf("Error reading file %s\n");
         else done++;
      }
      else if ( itmp < sizeof(s.data) )
      {
         for (i=itmp; i<sizeof(s.data); i++ ) s.data[i] = 0;
      }
      if ( !done ) 
      {
         int rc;
         if ( (rc = send_sp_packet(s)) < 0 ) 
            done = ++early;
         else if (rc == 0)
            ++done;
      }
   } while ( !done );

   if ( !early ) 
      send_eot();
// Sleep (1000);
// echo_chars();
   return 0;
}

//////////////////////////////////////////////////////////////////////////////
int send_sp_packet(sp_pkt& s1)
{
   char csum, i, chr;
   DWORD dret = 1;
   int sendcnt, done, rcvcnt, retval=0;
static int myCount = 0;

   // First, prepare the pkt data
   for ( i=0, csum=0; i<sizeof(s1.data); i++ )
      csum = (char)(csum + s1.data[i]);
   s1.start = SOH;
   s1.csum = csum;
   s1.serial = ++frame_num;
   // Flush all in the receive queue
// while (dret) ReadFile(hCom, &csum, 1, &dret, NULL);
myCount ++;
if (myCount % 256 != (unsigned char)s1.serial)
   printf ("\nmycount = %d s1.serial = %d\n", myCount, s1.serial);
   // Loop for implementing the handshake protocol
   sendcnt = done = 0;
   do 
   {
      // Send the packet
      if (!WriteFile(hCom, &s1, sizeof(s1), &dret, NULL))
      {
         printf("send_sp_packet: WriteFile Failed error = %ld\n",
                  GetLastError());
         ++done;
         retval = 0;
      }
      else if ( dret != sizeof(s1) )
      {
         printf("Error writing simple file packet frame\n");
         ++done;
         retval = 0;
      }
      else 
      {
//printf ("  W%4.4d", myCount);
         // Wait for send to finish, app 20 milliseconds
//       Sleep(0);
//Sleep(500);
         rcvcnt = 0;
         // Try (up to 10 times) to get a char from receiver.
         do 
         {
//printf ("R");
            if (!ReadFile(hCom, &chr, 1, &dret, NULL))
            {
               if ((dret == 0) && (GetLastError() == 995L))
               {
                  // Timeout?
                  Sleep(1000);
                  printf("X");
               }
               else
               {
                  printf("send_sp_packet:: ReadFile Failed error = %ld\n",
                           GetLastError());
                  ++done;
                  retval = 0;
               }
            }
               // If an ACK, we be done with this frame
            if ( dret != 1 ) 
               Sleep(2);
            else if ( chr == ACK ) 
            {
               printf("*");
               retval = ++done;
            }
            else if ( chr == NAK )
               printf("/");
            else if (chr == ESC )
            {
               done++;
               printf("|");
               echo_chars();
               retval = -1;
            }
            else 
{
//             printf("0x%2.2x ", chr);
printf("%c", chr);
echo_chars();
}
         } while ( ++rcvcnt < 10 && dret == 0 && !done );

         if (rcvcnt >= 10)
         {
            printf("\nsend_sp_packet:: ReadFile Failed error = %ld\n",
                     GetLastError());
            ++done;
            retval = 0;
         }
      }
   } while ( ++sendcnt < 5 && !done );
   return retval;
}
